@@ -10,16 +10,11 @@ from __future__ import print_function
 # sequence ::= SEQUENCE-START node* SEQUENCE-END
 # mapping ::= MAPPING-START (node node)* MAPPING-END
 
-__all__ = ['Emitter', 'EmitterError']
+from ruamel.yaml.error import YAMLError
+from ruamel.yaml.events import *                                     # NOQA
+from ruamel.yaml.compat import utf8, text_type, PY2, nprint, dbg, DBG_EVENT
 
-try:
-    from .error import YAMLError
-    from .events import *                                                # NOQA
-    from .compat import utf8, text_type, PY2, nprint, dbg, DBG_EVENT
-except (ImportError, ValueError):  # for Jython
-    from ruamel.yaml.error import YAMLError
-    from ruamel.yaml.events import *                                     # NOQA
-    from ruamel.yaml.compat import utf8, text_type, PY2, nprint, dbg, DBG_EVENT
+__all__ = ['Emitter', 'EmitterError']
 
 
 class EmitterError(YAMLError):
@@ -50,7 +45,8 @@ class Emitter(object):
     MAX_SIMPLE_KEY_LENGTH = 128
 
     def __init__(self, stream, canonical=None, indent=None, width=None,
-                 allow_unicode=None, line_break=None, block_seq_indent=None):
+                 allow_unicode=None, line_break=None, block_seq_indent=None,
+                 top_level_colon_align=None, prefix_colon=None):
 
         # The stream should have the methods `write` and possibly `flush`.
         self.stream = stream
@@ -89,19 +85,25 @@ class Emitter(object):
         self.column = 0
         self.whitespace = True
         self.indention = True
+        self.no_newline = None  # set if directly after `- `
 
         # Whether the document requires an explicit document indicator
         self.open_ended = False
 
+        # colon handling
+        self.colon = u':'
+        self.prefixed_colon = self.colon if prefix_colon is None else prefix_colon + self.colon
+
         # Formatting details.
         self.canonical = canonical
         self.allow_unicode = allow_unicode
         self.block_seq_indent = block_seq_indent if block_seq_indent else 0
+        self.top_level_colon_align = top_level_colon_align
         self.best_indent = 2
         if indent and 1 < indent < 10:
             self.best_indent = indent
-        if self.best_indent < self.block_seq_indent + 1:
-            self.best_indent = self.block_seq_indent + 1
+        # if self.best_indent < self.block_seq_indent + 1:
+        #     self.best_indent = self.block_seq_indent + 1
         self.best_width = 80
         if width and width > self.best_indent*2:
             self.best_width = width
@@ -162,7 +164,7 @@ class Emitter(object):
                 return False
         return (len(self.events) < count+1)
 
-    def increase_indent(self, flow=False, indentless=False):
+    def increase_indent(self, flow=False, sequence=None, indentless=False):
         self.indents.append(self.indent)
         if self.indent is None:
             if flow:
@@ -171,6 +173,8 @@ class Emitter(object):
                 self.indent = 0
         elif not indentless:
             self.indent += self.best_indent
+            # if self.sequence_context and (self.block_seq_indent + 2) > self.best_indent:
+            #    self.indent = self.block_seq_indent + 2
 
     # States.
 
@@ -311,7 +315,7 @@ class Emitter(object):
     def expect_flow_sequence(self):
         self.write_indicator(u'[', True, whitespace=True)
         self.flow_level += 1
-        self.increase_indent(flow=True)
+        self.increase_indent(flow=True, sequence=True)
         self.state = self.expect_first_flow_sequence_item
 
     def expect_first_flow_sequence_item(self):
@@ -350,7 +354,7 @@ class Emitter(object):
     def expect_flow_mapping(self):
         self.write_indicator(u'{', True, whitespace=True)
         self.flow_level += 1
-        self.increase_indent(flow=True)
+        self.increase_indent(flow=True, sequence=False)
         self.state = self.expect_first_flow_mapping_key
 
     def expect_first_flow_mapping_key(self):
@@ -400,14 +404,14 @@ class Emitter(object):
                 self.expect_node(mapping=True)
 
     def expect_flow_mapping_simple_value(self):
-        self.write_indicator(u':', False)
+        self.write_indicator(self.prefixed_colon, False)
         self.states.append(self.expect_flow_mapping_key)
         self.expect_node(mapping=True)
 
     def expect_flow_mapping_value(self):
         if self.canonical or self.column > self.best_width:
             self.write_indent()
-        self.write_indicator(u':', True)
+        self.write_indicator(self.prefixed_colon, True)
         self.states.append(self.expect_flow_mapping_key)
         self.expect_node(mapping=True)
 
@@ -415,7 +419,7 @@ class Emitter(object):
 
     def expect_block_sequence(self):
         indentless = (self.mapping_context and not self.indention)
-        self.increase_indent(flow=False, indentless=indentless)
+        self.increase_indent(flow=False, sequence=True, indentless=indentless)
         self.state = self.expect_first_block_sequence_item
 
     def expect_first_block_sequence_item(self):
@@ -434,13 +438,15 @@ class Emitter(object):
                 self.write_pre_comment(self.event)
             self.write_indent()
             self.write_indicator((u' ' * self.block_seq_indent) + u'-', True, indention=True)
+            if self.block_seq_indent + 2 > self.best_indent:
+                self.no_newline = True
             self.states.append(self.expect_block_sequence_item)
             self.expect_node(sequence=True)
 
     # Block mapping handlers.
 
     def expect_block_mapping(self):
-        self.increase_indent(flow=False)
+        self.increase_indent(flow=False, sequence=False)
         self.state = self.expect_first_block_mapping_key
 
     def expect_first_block_mapping_key(self):
@@ -459,8 +465,9 @@ class Emitter(object):
                 self.write_pre_comment(self.event)
             self.write_indent()
             if self.check_simple_key():
-                if self.event.style == '?':
-                    self.write_indicator(u'?', True, indention=True)
+                if not isinstance(self.event, SequenceStartEvent):  # sequence keys
+                    if self.event.style == '?':
+                        self.write_indicator(u'?', True, indention=True)
                 self.states.append(self.expect_block_mapping_simple_value)
                 self.expect_node(mapping=True, simple_key=True)
             else:
@@ -470,13 +477,19 @@ class Emitter(object):
 
     def expect_block_mapping_simple_value(self):
         if getattr(self.event, 'style', None) != '?':
-            self.write_indicator(u':', False)
+            # prefix = u''
+            if self.indent == 0 and self.top_level_colon_align is not None:
+                # write non-prefixed colon
+                c = u' ' * (self.top_level_colon_align - self.column) + self.colon
+            else:
+                c = self.prefixed_colon
+            self.write_indicator(c, False)
         self.states.append(self.expect_block_mapping_key)
         self.expect_node(mapping=True)
 
     def expect_block_mapping_value(self):
         self.write_indent()
-        self.write_indicator(u':', True, indention=True)
+        self.write_indicator(self.prefixed_colon, True, indention=True)
         self.states.append(self.expect_block_mapping_key)
         self.expect_node(mapping=True)
 
@@ -514,6 +527,8 @@ class Emitter(object):
             length += len(self.analysis.scalar)
         return (length < self.MAX_SIMPLE_KEY_LENGTH and (
             isinstance(self.event, AliasEvent) or
+            (isinstance(self.event, SequenceStartEvent) and
+             self.event.flow_style is True) or
             (isinstance(self.event, ScalarEvent) and
              not self.analysis.empty and not self.analysis.multiline) or
             self.check_empty_sequence() or self.check_empty_mapping()))
@@ -886,7 +901,10 @@ class Emitter(object):
         indent = self.indent or 0
         if not self.indention or self.column > indent   \
                 or (self.column == indent and not self.whitespace):
-            self.write_line_break()
+            if self.no_newline:
+                self.no_newline = False
+            else:
+                self.write_line_break()
         if self.column < indent:
             self.whitespace = True
             data = u' '*(indent-self.column)
@@ -1228,8 +1246,8 @@ class Emitter(object):
                 nr_spaces = 1
             value = ' ' * nr_spaces + value
             try:
-                if PY2:
-                    value = value.encode('utf-8')
+                if self.encoding:
+                    value = value.encode(self.encoding)
             except UnicodeDecodeError:
                 pass
             self.stream.write(value)
