@@ -9,6 +9,7 @@ from ruamel.yaml.error import *                       # NOQA
 from ruamel.yaml.nodes import *                       # NOQA
 from ruamel.yaml.compat import text_type, binary_type, to_unicode, PY2, PY3, ordereddict
 from ruamel.yaml.scalarstring import *                # NOQA
+from ruamel.yaml.timestamp import TimeStamp
 
 import datetime
 import sys
@@ -575,8 +576,8 @@ Representer.add_multi_representer(object,
                                   Representer.represent_object)
 
 
-from ruamel.yaml.comments import CommentedMap, CommentedOrderedMap, \
-    CommentedSeq, CommentedSet, comment_attrib, merge_attrib    # NOQA
+from ruamel.yaml.comments import CommentedMap, CommentedOrderedMap, CommentedSeq, \
+    CommentedKeySeq, CommentedSet, comment_attrib, merge_attrib    # NOQA
 
 
 class RoundTripRepresenter(SafeRepresenter):
@@ -657,6 +658,12 @@ class RoundTripRepresenter(SafeRepresenter):
                 node.flow_style = best_style
         return node
 
+    def represent_key(self, data):
+        if isinstance(data, CommentedKeySeq):
+            return self.represent_sequence(u'tag:yaml.org,2002:seq', data,
+                                           flow_style=True)
+        return SafeRepresenter.represent_key(self, data)
+
     def represent_mapping(self, tag, mapping, flow_style=None):
         value = []
         try:
@@ -689,7 +696,12 @@ class RoundTripRepresenter(SafeRepresenter):
                 pass
         except AttributeError:
             item_comments = {}
-        for item_key, item_value in mapping.items():
+        merge_list = [m[1] for m in getattr(mapping, merge_attrib, [])]
+        if merge_list:
+            items = mapping.non_merged_items()
+        else:
+            items = mapping.items()
+        for item_key, item_value in items:
             node_key = self.represent_key(item_key)
             node_value = self.represent_data(item_value)
             item_comment = item_comments.get(item_key)
@@ -713,7 +725,6 @@ class RoundTripRepresenter(SafeRepresenter):
                 node.flow_style = self.default_flow_style
             else:
                 node.flow_style = best_style
-        merge_list = [m[1] for m in getattr(mapping, merge_attrib, [])]
         if merge_list:
             # because of the call to represent_data here, the anchors
             # are marked as being used and thereby created
@@ -847,6 +858,18 @@ class RoundTripRepresenter(SafeRepresenter):
             tag = u'tag:yaml.org,2002:map'
         return self.represent_mapping(tag, data)
 
+    def represent_datetime(self, data):
+        inter = 'T' if data._yaml['t'] else ' '
+        _yaml = data._yaml
+        if _yaml['delta']:
+            data += _yaml['delta']
+            value = data.isoformat(inter)
+        else:
+            value = data.isoformat(inter)
+        if _yaml['tz']:
+            value += _yaml['tz']
+        return self.represent_scalar(u'tag:yaml.org,2002:timestamp', to_unicode(value))
+
 
 RoundTripRepresenter.add_representer(type(None),
                                      RoundTripRepresenter.represent_none)
@@ -879,3 +902,6 @@ if sys.version_info >= (2, 7):
 
 RoundTripRepresenter.add_representer(CommentedSet,
                                      RoundTripRepresenter.represent_set)
+
+RoundTripRepresenter.add_representer(TimeStamp,
+                                     RoundTripRepresenter.represent_datetime)
